@@ -1,36 +1,47 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package mongodb
 
 import (
 	"context"
 	"crypto/tls"
 	"crypto/x509"
+	_ "embed"
 	"fmt"
 	"net/url"
 	"strings"
 	"sync"
 	"time"
 
-	"github.com/influxdata/telegraf"
-	tlsint "github.com/influxdata/telegraf/plugins/common/tls"
-	"github.com/influxdata/telegraf/plugins/inputs"
 	"go.mongodb.org/mongo-driver/mongo"
 	"go.mongodb.org/mongo-driver/mongo/options"
 	"go.mongodb.org/mongo-driver/mongo/readpref"
+
+	"github.com/influxdata/telegraf"
+	"github.com/influxdata/telegraf/internal/choice"
+	tlsint "github.com/influxdata/telegraf/plugins/common/tls"
+	"github.com/influxdata/telegraf/plugins/inputs"
 )
 
+//go:embed sample.conf
+var sampleConfig string
+
+var DisconnectedServersBehaviors = []string{"error", "skip"}
+
 type MongoDB struct {
-	Servers             []string
-	Ssl                 Ssl
-	GatherClusterStatus bool
-	GatherPerdbStats    bool
-	GatherColStats      bool
-	GatherTopStat       bool
-	ColStatsDbs         []string
+	Servers                     []string
+	Ssl                         Ssl
+	GatherClusterStatus         bool
+	GatherPerdbStats            bool
+	GatherColStats              bool
+	GatherTopStat               bool
+	DisconnectedServersBehavior string
+	ColStatsDbs                 []string
 	tlsint.ClientConfig
 
 	Log telegraf.Logger `toml:"-"`
 
-	clients []*Server
+	clients   []*Server
+	tlsConfig *tls.Config
 }
 
 type Ssl struct {
@@ -38,54 +49,22 @@ type Ssl struct {
 	CaCerts []string `toml:"cacerts" deprecated:"1.3.0;use 'tls_ca' instead"`
 }
 
-var sampleConfig = `
-  ## An array of URLs of the form:
-  ##   "mongodb://" [user ":" pass "@"] host [ ":" port]
-  ## For example:
-  ##   mongodb://user:auth_key@10.10.3.30:27017,
-  ##   mongodb://10.10.3.33:18832,
-  servers = ["mongodb://127.0.0.1:27017?connect=direct"]
-
-  ## When true, collect cluster status
-  ## Note that the query that counts jumbo chunks triggers a COLLSCAN, which
-  ## may have an impact on performance.
-  # gather_cluster_status = true
-
-  ## When true, collect per database stats
-  # gather_perdb_stats = false
-
-  ## When true, collect per collection stats
-  # gather_col_stats = false
-
-  ## When true, collect usage statistics for each collection
-  ## (insert, update, queries, remove, getmore, commands etc...).
-  # gather_top_stat = false
-
-  ## List of db where collections stats are collected
-  ## If empty, all db are concerned
-  # col_stats_dbs = ["local"]
-
-  ## Optional TLS Config
-  # tls_ca = "/etc/telegraf/ca.pem"
-  # tls_cert = "/etc/telegraf/cert.pem"
-  # tls_key = "/etc/telegraf/key.pem"
-  ## Use TLS but skip chain & host verification
-  # insecure_skip_verify = false
-`
-
-func (m *MongoDB) SampleConfig() string {
+func (*MongoDB) SampleConfig() string {
 	return sampleConfig
 }
 
-func (*MongoDB) Description() string {
-	return "Read metrics from one or many MongoDB servers"
-}
-
 func (m *MongoDB) Init() error {
-	var tlsConfig *tls.Config
+	if m.DisconnectedServersBehavior == "" {
+		m.DisconnectedServersBehavior = "error"
+	}
+
+	if err := choice.Check(m.DisconnectedServersBehavior, DisconnectedServersBehaviors); err != nil {
+		return fmt.Errorf("disconnected_servers_behavior: %w", err)
+	}
+
 	if m.Ssl.Enabled {
 		// Deprecated TLS config
-		tlsConfig = &tls.Config{
+		m.tlsConfig = &tls.Config{
 			InsecureSkipVerify: m.ClientConfig.InsecureSkipVerify,
 		}
 		if len(m.Ssl.CaCerts) == 0 {
@@ -98,10 +77,10 @@ func (m *MongoDB) Init() error {
 				return fmt.Errorf("failed to parse root certificate")
 			}
 		}
-		tlsConfig.RootCAs = roots
+		m.tlsConfig.RootCAs = roots
 	} else {
 		var err error
-		tlsConfig, err = m.ClientConfig.TLSConfig()
+		m.tlsConfig, err = m.ClientConfig.TLSConfig()
 		if err != nil {
 			return err
 		}
@@ -111,6 +90,11 @@ func (m *MongoDB) Init() error {
 		m.Servers = []string{"mongodb://127.0.0.1:27017"}
 	}
 
+	return nil
+}
+
+// Start runs after init and setup mongodb connections
+func (m *MongoDB) Start() error {
 	for _, connURL := range m.Servers {
 		if !strings.HasPrefix(connURL, "mongodb://") && !strings.HasPrefix(connURL, "mongodb+srv://") {
 			// Preserve backwards compatibility for hostnames without a
@@ -121,15 +105,15 @@ func (m *MongoDB) Init() error {
 
 		u, err := url.Parse(connURL)
 		if err != nil {
-			return fmt.Errorf("unable to parse connection URL: %q", err)
+			return fmt.Errorf("unable to parse connection URL: %w", err)
 		}
 
 		ctx, cancel := context.WithTimeout(context.Background(), 10*time.Second)
 		defer cancel() //nolint:revive
 
 		opts := options.Client().ApplyURI(connURL)
-		if tlsConfig != nil {
-			opts.TLSConfig = tlsConfig
+		if m.tlsConfig != nil {
+			opts.TLSConfig = m.tlsConfig
 		}
 		if opts.ReadPreference == nil {
 			opts.ReadPreference = readpref.Nearest()
@@ -137,12 +121,16 @@ func (m *MongoDB) Init() error {
 
 		client, err := mongo.Connect(ctx, opts)
 		if err != nil {
-			return fmt.Errorf("unable to connect to MongoDB: %q", err)
+			return fmt.Errorf("unable to connect to MongoDB: %w", err)
 		}
 
 		err = client.Ping(ctx, opts.ReadPreference)
 		if err != nil {
-			return fmt.Errorf("unable to connect to MongoDB: %s", err)
+			if m.DisconnectedServersBehavior == "error" {
+				return fmt.Errorf("unable to ping MongoDB: %w", err)
+			}
+
+			m.Log.Errorf("unable to ping MongoDB: %w", err)
 		}
 
 		server := &Server{
@@ -164,9 +152,16 @@ func (m *MongoDB) Gather(acc telegraf.Accumulator) error {
 		wg.Add(1)
 		go func(srv *Server) {
 			defer wg.Done()
+			if m.DisconnectedServersBehavior == "skip" {
+				if err := srv.ping(); err != nil {
+					m.Log.Debugf("failed to ping server: %w", err)
+					return
+				}
+			}
+
 			err := srv.gatherData(acc, m.GatherClusterStatus, m.GatherPerdbStats, m.GatherColStats, m.GatherTopStat, m.ColStatsDbs)
 			if err != nil {
-				m.Log.Errorf("failed to gather data: %q", err)
+				m.Log.Errorf("failed to gather data: %w", err)
 			}
 		}(client)
 	}
