@@ -1,33 +1,41 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package wavefront
 
 import (
+	_ "embed"
 	"fmt"
+	"net/url"
 	"regexp"
 	"strings"
 
+	wavefront "github.com/wavefronthq/wavefront-sdk-go/senders"
+
 	"github.com/influxdata/telegraf"
 	"github.com/influxdata/telegraf/plugins/outputs"
-	wavefront "github.com/wavefronthq/wavefront-sdk-go/senders"
 )
 
+//go:embed sample.conf
+var sampleConfig string
+
 const maxTagLength = 254
 
 type Wavefront struct {
-	URL             string                          `toml:"url"`
-	Token           string                          `toml:"token"`
-	Host            string                          `toml:"host"`
-	Port            int                             `toml:"port"`
-	Prefix          string                          `toml:"prefix"`
-	SimpleFields    bool                            `toml:"simple_fields"`
-	MetricSeparator string                          `toml:"metric_separator"`
-	ConvertPaths    bool                            `toml:"convert_paths"`
-	ConvertBool     bool                            `toml:"convert_bool"`
-	UseRegex        bool                            `toml:"use_regex"`
-	UseStrict       bool                            `toml:"use_strict"`
-	TruncateTags    bool                            `toml:"truncate_tags"`
-	ImmediateFlush  bool                            `toml:"immediate_flush"`
-	SourceOverride  []string                        `toml:"source_override"`
-	StringToNumber  map[string][]map[string]float64 `toml:"string_to_number"`
+	URL                  string                          `toml:"url"`
+	Token                string                          `toml:"token"`
+	Host                 string                          `toml:"host" deprecated:"2.4.0;use url instead"`
+	Port                 int                             `toml:"port" deprecated:"2.4.0;use url instead"`
+	Prefix               string                          `toml:"prefix"`
+	SimpleFields         bool                            `toml:"simple_fields"`
+	MetricSeparator      string                          `toml:"metric_separator"`
+	ConvertPaths         bool                            `toml:"convert_paths"`
+	ConvertBool          bool                            `toml:"convert_bool"`
+	HTTPMaximumBatchSize int                             `toml:"http_maximum_batch_size"`
+	UseRegex             bool                            `toml:"use_regex"`
+	UseStrict            bool                            `toml:"use_strict"`
+	TruncateTags         bool                            `toml:"truncate_tags"`
+	ImmediateFlush       bool                            `toml:"immediate_flush"`
+	SourceOverride       []string                        `toml:"source_override"`
+	StringToNumber       map[string][]map[string]float64 `toml:"string_to_number" deprecated:"1.9.0;use the enum processor instead"`
 
 	sender wavefront.Sender
 	Log    telegraf.Logger `toml:"-"`
@@ -51,71 +59,12 @@ var strictSanitizedChars = strings.NewReplacer(
 )
 
 // instead of Replacer which may miss some special characters we can use a regex pattern, but this is significantly slower than Replacer
-var sanitizedRegex = regexp.MustCompile("[^a-zA-Z\\d_.-]")
+var sanitizedRegex = regexp.MustCompile(`[^a-zA-Z\d_.-]`)
 
 var tagValueReplacer = strings.NewReplacer("*", "-")
 
 var pathReplacer = strings.NewReplacer("_", "_")
 
-var sampleConfig = `
-  ## Url for Wavefront Direct Ingestion or using HTTP with Wavefront Proxy
-  ## If using Wavefront Proxy, also specify port. example: http://proxyserver:2878
-  url = "https://metrics.wavefront.com"
-
-  ## Authentication Token for Wavefront. Only required if using Direct Ingestion
-  #token = "DUMMY_TOKEN"  
-  
-  ## DNS name of the wavefront proxy server. Do not use if url is specified
-  #host = "wavefront.example.com"
-
-  ## Port that the Wavefront proxy server listens on. Do not use if url is specified
-  #port = 2878
-
-  ## prefix for metrics keys
-  #prefix = "my.specific.prefix."
-
-  ## whether to use "value" for name of simple fields. default is false
-  #simple_fields = false
-
-  ## character to use between metric and field name.  default is . (dot)
-  #metric_separator = "."
-
-  ## Convert metric name paths to use metricSeparator character
-  ## When true will convert all _ (underscore) characters in final metric name. default is true
-  #convert_paths = true
-
-  ## Use Strict rules to sanitize metric and tag names from invalid characters
-  ## When enabled forward slash (/) and comma (,) will be accepted
-  #use_strict = false
-
-  ## Use Regex to sanitize metric and tag names from invalid characters
-  ## Regex is more thorough, but significantly slower. default is false
-  #use_regex = false
-
-  ## point tags to use as the source name for Wavefront (if none found, host will be used)
-  #source_override = ["hostname", "address", "agent_host", "node_host"]
-
-  ## whether to convert boolean values to numeric values, with false -> 0.0 and true -> 1.0. default is true
-  #convert_bool = true
-
-  ## Truncate metric tags to a total of 254 characters for the tag name value. Wavefront will reject any 
-  ## data point exceeding this limit if not truncated. Defaults to 'false' to provide backwards compatibility.
-  #truncate_tags = false
-
-  ## Flush the internal buffers after each batch. This effectively bypasses the background sending of metrics
-  ## normally done by the Wavefront SDK. This can be used if you are experiencing buffer overruns. The sending 
-  ## of metrics will block for a longer time, but this will be handled gracefully by the internal buffering in
-  ## Telegraf.
-  #immediate_flush = true
-
-  ## Define a mapping, namespaced by metric prefix, from string values to numeric values
-  ##   deprecated in 1.9; use the enum processor plugin
-  #[[outputs.wavefront.string_to_number.elasticsearch]]
-  #  green = 1.0
-  #  yellow = 0.5
-  #  red = 0.0
-`
-
 type MetricPoint struct {
 	Metric    string
 	Value     float64
@@ -124,39 +73,54 @@ type MetricPoint struct {
 	Tags      map[string]string
 }
 
-func (w *Wavefront) Connect() error {
-	if len(w.StringToNumber) > 0 {
-		w.Log.Warn("The string_to_number option is deprecated; please use the enum processor instead")
-	}
+func (*Wavefront) SampleConfig() string {
+	return sampleConfig
+}
 
+func senderURLFromURLAndToken(rawURL, token string) (string, error) {
+	newURL, err := url.Parse(rawURL)
+	if err != nil {
+		return "", fmt.Errorf("could not parse the provided Url: %s", rawURL)
+	}
+	newURL.User = url.User(token)
+
+	return newURL.String(), nil
+}
+
+func senderURLFromHostAndPort(host string, port int) string {
+	return fmt.Sprintf("http://%s:%d", host, port)
+}
+
+func (w *Wavefront) Connect() error {
 	flushSeconds := 5
 	if w.ImmediateFlush {
 		flushSeconds = 86400 // Set a very long flush interval if we're flushing directly
 	}
+	var connectionURL string
 	if w.URL != "" {
 		w.Log.Debug("connecting over http/https using Url: %s", w.URL)
-		sender, err := wavefront.NewDirectSender(&wavefront.DirectConfiguration{
-			Server:               w.URL,
-			Token:                w.Token,
-			FlushIntervalSeconds: flushSeconds,
-		})
+		connectionURLWithToken, err := senderURLFromURLAndToken(w.URL, w.Token)
 		if err != nil {
-			return fmt.Errorf("could not create Wavefront Sender for Url: %s", w.URL)
+			return err
 		}
-		w.sender = sender
+		connectionURL = connectionURLWithToken
 	} else {
-		w.Log.Debugf("connecting over tcp using Host: %q and Port: %d", w.Host, w.Port)
-		sender, err := wavefront.NewProxySender(&wavefront.ProxyConfiguration{
-			Host:                 w.Host,
-			MetricsPort:          w.Port,
-			FlushIntervalSeconds: flushSeconds,
-		})
-		if err != nil {
-			return fmt.Errorf("could not create Wavefront Sender for Host: %q and Port: %d", w.Host, w.Port)
-		}
-		w.sender = sender
+		w.Log.Warnf("configuration with host/port is deprecated. Please use url.")
+		w.Log.Debugf("connecting over http using Host: %q and Port: %d", w.Host, w.Port)
+		connectionURL = senderURLFromHostAndPort(w.Host, w.Port)
 	}
 
+	sender, err := wavefront.NewSender(connectionURL,
+		wavefront.BatchSize(w.HTTPMaximumBatchSize),
+		wavefront.FlushIntervalSeconds(flushSeconds),
+	)
+
+	if err != nil {
+		return fmt.Errorf("could not create Wavefront Sender for the provided url")
+	}
+
+	w.sender = sender
+
 	if w.ConvertPaths && w.MetricSeparator == "_" {
 		w.ConvertPaths = false
 	}
@@ -172,6 +136,9 @@ func (w *Wavefront) Write(metrics []telegraf.Metric) error {
 			err := w.sender.SendMetric(point.Metric, point.Value, point.Timestamp, point.Source, point.Tags)
 			if err != nil {
 				if isRetryable(err) {
+					if flushErr := w.sender.Flush(); flushErr != nil {
+						w.Log.Errorf("wavefront flushing error: %v", flushErr)
+					}
 					return fmt.Errorf("wavefront sending error: %v", err)
 				}
 				w.Log.Errorf("non-retryable error during Wavefront.Write: %v", err)
@@ -187,7 +154,7 @@ func (w *Wavefront) Write(metrics []telegraf.Metric) error {
 }
 
 func (w *Wavefront) buildMetrics(m telegraf.Metric) []*MetricPoint {
-	ret := []*MetricPoint{}
+	ret := make([]*MetricPoint, 0)
 
 	for fieldName, value := range m.Fields() {
 		var name string
@@ -249,7 +216,10 @@ func (w *Wavefront) buildTags(mTags map[string]string) (string, map[string]strin
 			for k, v := range mTags {
 				if k == s {
 					source = v
-					mTags["telegraf_host"] = mTags["host"]
+					if mTags["host"] != "" {
+						mTags["telegraf_host"] = mTags["host"]
+					}
+
 					sourceTagFound = true
 					delete(mTags, k)
 					break
@@ -330,14 +300,6 @@ func buildValue(v interface{}, name string, w *Wavefront) (float64, error) {
 	return 0, fmt.Errorf("unexpected type: %T, with value: %v, for: %s", v, v, name)
 }
 
-func (w *Wavefront) SampleConfig() string {
-	return sampleConfig
-}
-
-func (w *Wavefront) Description() string {
-	return "Configuration for Wavefront server to send metrics to"
-}
-
 func (w *Wavefront) Close() error {
 	w.sender.Close()
 	return nil
@@ -346,12 +308,13 @@ func (w *Wavefront) Close() error {
 func init() {
 	outputs.Add("wavefront", func() telegraf.Output {
 		return &Wavefront{
-			Token:           "DUMMY_TOKEN",
-			MetricSeparator: ".",
-			ConvertPaths:    true,
-			ConvertBool:     true,
-			TruncateTags:    false,
-			ImmediateFlush:  true,
+			Token:                "DUMMY_TOKEN",
+			MetricSeparator:      ".",
+			ConvertPaths:         true,
+			ConvertBool:          true,
+			TruncateTags:         false,
+			ImmediateFlush:       true,
+			HTTPMaximumBatchSize: 10000,
 		}
 	})
 }
